@@ -3,8 +3,10 @@
  *
  * Copyright (C) 2003  CodeFactory AB
  *
+ * SPDX-License-Identifier: AFL-2.1 OR GPL-2.0-or-later
+ *
  * Licensed under the Academic Free License version 2.1
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -14,7 +16,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -34,7 +36,8 @@
 #define DBUS_SERVICE_NAME     "Name"
 #define DBUS_SERVICE_EXEC     "Exec"
 #define DBUS_SERVICE_USER     "User"
-#define DBUS_SERVICE_GROUP    "Group"
+#define DBUS_SERVICE_SYSTEMD_SERVICE "SystemdService"
+#define DBUS_SERVICE_ASSUMED_APPARMOR_LABEL "AssumedAppArmorLabel"
 
 typedef struct BusDesktopFile BusDesktopFile;
 
@@ -52,5 +55,10 @@ dbus_bool_t bus_desktop_file_get_string (BusDesktopFile  *desktop_file,
 					 char           **val,
 					 DBusError       *error);
 
+static inline void
+bus_clear_desktop_file (BusDesktopFile **desktop_p)
+{
+  _dbus_clear_pointer_impl (BusDesktopFile, desktop_p, bus_desktop_file_free);
+}
 
 #endif /* BUS_DESKTOP_FILE_H */
