@@ -3,6 +3,8 @@
  *
  * Copyright (C) 2005  Red Hat, Inc.
  *
+ * SPDX-License-Identifier: AFL-2.1 OR GPL-2.0-or-later
+ *
  * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,11 +26,7 @@
 #ifndef DBUS_MARSHAL_VALIDATE_H
 #define DBUS_MARSHAL_VALIDATE_H
 
-#include <config.h>
-
-#ifndef PACKAGE
-#error "config.h not included here"
-#endif
+#include <dbus/dbus-string.h>
 
 /**
  * @addtogroup DBusMarshal
@@ -117,12 +115,16 @@ typedef enum
   DBUS_INVALID_DICT_ENTRY_HAS_TOO_MANY_FIELDS = 53,
   DBUS_INVALID_DICT_ENTRY_NOT_INSIDE_ARRAY = 54,
   DBUS_INVALID_DICT_KEY_MUST_BE_BASIC_TYPE = 55,
+  DBUS_INVALID_MISSING_UNIX_FDS = 56,
+  DBUS_INVALID_NESTED_TOO_DEEPLY = 57,
   DBUS_VALIDITY_LAST
 } DBusValidity;
 
+DBUS_PRIVATE_EXPORT
 DBusValidity _dbus_validate_signature_with_reason (const DBusString *type_str,
                                                    int               type_pos,
                                                    int               len);
+DBUS_PRIVATE_EXPORT
 DBusValidity _dbus_validate_body_with_reason      (const DBusString *expected_signature,
                                                    int               expected_signature_start,
                                                    int               byte_order,
@@ -133,24 +135,32 @@ DBusValidity _dbus_validate_body_with_reason      (const DBusString *expected_si
 
 const char *_dbus_validity_to_error_message (DBusValidity validity);
 
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_validate_path       (const DBusString *str,
                                        int               start,
                                        int               len);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_validate_interface  (const DBusString *str,
                                        int               start,
                                        int               len);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_validate_member     (const DBusString *str,
                                        int               start,
                                        int               len);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_validate_error_name (const DBusString *str,
                                        int               start,
                                        int               len);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_validate_bus_name   (const DBusString *str,
                                        int               start,
                                        int               len);
-dbus_bool_t _dbus_validate_signature  (const DBusString *str,
-                                       int               start,
-                                       int               len);
+DBUS_PRIVATE_EXPORT
+dbus_bool_t _dbus_validate_bus_namespace (const DBusString  *str,
+                                          int                start,
+                                          int                len);
+/* just to have a name consistent with the above: */
+#define _dbus_validate_utf8(s,b,e) _dbus_string_validate_utf8 (s, b, e)
 
 #ifdef DBUS_DISABLE_CHECKS
 
@@ -195,8 +205,8 @@ DECLARE_DBUS_NAME_CHECK(member);
 DECLARE_DBUS_NAME_CHECK(error_name);
 /** defines _dbus_check_is_valid_bus_name() */
 DECLARE_DBUS_NAME_CHECK(bus_name);
-/** defines _dbus_check_is_valid_signature() */
-DECLARE_DBUS_NAME_CHECK(signature);
+/** defines _dbus_check_is_valid_utf8() */
+DECLARE_DBUS_NAME_CHECK(utf8);
 
 /** @} */
 
