@@ -3,6 +3,8 @@
  *
  * Copyright (C) 2003, 2005  Red Hat Inc.
  *
+ * SPDX-License-Identifier: AFL-2.1 OR GPL-2.0-or-later
+ *
  * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,6 +30,7 @@
 #include "dbus-hash.h"
 #include "dbus-protocol.h"
 #include "dbus-string.h"
+#include <dbus/dbus-test-tap.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -319,6 +322,7 @@ find_subtree_recurse (DBusObjectSubtree  *subtree,
     }
 }
 
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 static DBusObjectSubtree*
 find_subtree (DBusObjectTree *tree,
               const char    **path,
@@ -337,6 +341,7 @@ find_subtree (DBusObjectTree *tree,
   else
     return subtree;
 }
+#endif
 
 static DBusObjectSubtree*
 lookup_subtree (DBusObjectTree *tree,
@@ -433,6 +438,185 @@ _dbus_object_tree_register (DBusObjectTree              *tree,
   return TRUE;
 }
 
+/**
+ * Attempts to unregister the given subtree.  If the subtree is registered,
+ * stores its unregister function and user data for later use and returns
+ * #TRUE.  If subtree is not registered, simply returns #FALSE.  Does not free
+ * subtree or remove it from the object tree.
+ *
+ * @param subtree the subtree to unregister
+ * @param unregister_function_out stores subtree's unregister_function
+ * @param user_data_out stores subtree's user_data
+ * @return #FALSE if the subtree was not registered, #TRUE on success
+ */
+static dbus_bool_t
+unregister_subtree (DBusObjectSubtree                 *subtree,
+                    DBusObjectPathUnregisterFunction  *unregister_function_out,
+                    void                             **user_data_out)
+{
+  _dbus_assert (subtree != NULL);
+  _dbus_assert (unregister_function_out != NULL);
+  _dbus_assert (user_data_out != NULL);
+
+  /* Confirm subtree is registered */
+  if (subtree->message_function != NULL)
+    {
+      subtree->message_function = NULL;
+
+      *unregister_function_out = subtree->unregister_function;
+      *user_data_out = subtree->user_data;
+
+      subtree->unregister_function = NULL;
+      subtree->user_data = NULL;
+
+      return TRUE;
+    }
+  else
+    {
+      /* Assert that this unregistered subtree is either the root node or has
+         children, otherwise we have a dangling path which should never
+         happen */
+      _dbus_assert (subtree->parent == NULL || subtree->n_subtrees > 0);
+
+      /* The subtree is not registered */
+      return FALSE;
+    }
+}
+
+/**
+ * Attempts to remove a child subtree from its parent.  If removal is
+ * successful, also frees the child.  Returns #TRUE on success, #FALSE
+ * otherwise.  A #FALSE return value tells unregister_and_free_path_recurse to
+ * stop attempting to remove ancestors, i.e., that no ancestors of the
+ * specified child are eligible for removal.
+ *
+ * @param parent parent from which to remove child
+ * @param child_index parent->subtrees index of child to remove
+ * @return #TRUE if removal and free succeed, #FALSE otherwise
+ */
+static dbus_bool_t
+attempt_child_removal (DBusObjectSubtree  *parent,
+                       int child_index)
+{
+  /* Candidate for removal */
+  DBusObjectSubtree* candidate;
+
+  _dbus_assert (parent != NULL);
+  _dbus_assert (child_index >= 0 && child_index < parent->n_subtrees);
+
+  candidate = parent->subtrees[child_index];
+  _dbus_assert (candidate != NULL);
+
+  if (candidate->n_subtrees == 0 && candidate->message_function == NULL)
+    {
+      /* The candidate node is childless and is not a registered
+         path, so... */
+
+      /* ... remove it from its parent... */
+      /* Assumes a 0-byte memmove is OK */
+      memmove (&parent->subtrees[child_index],
+               &parent->subtrees[child_index + 1],
+               (parent->n_subtrees - child_index - 1)
+               * sizeof (parent->subtrees[0]));
+      parent->n_subtrees -= 1;
+
+      /* ... and free it */
+      candidate->parent = NULL;
+      _dbus_object_subtree_unref (candidate);
+
+      return TRUE;
+    }
+  return FALSE;
+}
+
+/**
+ * Searches the object tree for a registered subtree node at the given path.
+ * If a registered node is found, it is removed from the tree and freed, and
+ * TRUE is returned.  If a registered subtree node is not found at the given
+ * path, the tree is not modified and FALSE is returned.
+ *
+ * The found node's unregister_function and user_data are returned in the
+ * corresponding _out arguments.  The caller should define these variables and
+ * pass their addresses as arguments.
+ *
+ * Likewise, the caller should define and set to TRUE a boolean variable, then
+ * pass its address as the continue_removal_attempts argument.
+ *
+ * Once a matching registered node is found, removed and freed, the recursive
+ * return path is traversed.  Along the way, eligible ancestor nodes are
+ * removed and freed.  An ancestor node is eligible for removal if and only if
+ * 1) it has no children, i.e., it has become childless and 2) it is not itself
+ * a registered handler.
+ *
+ * For example, suppose /A/B and /A/C are registered paths, and that these are
+ * the only paths in the tree.  If B is removed and freed, C is still reachable
+ * through A, so A cannot be removed and freed.  If C is subsequently removed
+ * and freed, then A becomes a childless node and it becomes eligible for
+ * removal, and will be removed and freed.
+ *
+ * Similarly, suppose /A is a registered path, and /A/B is also a registered
+ * path, and that these are the only paths in the tree.  If B is removed and
+ * freed, then even though A has become childless, it can't be freed because it
+ * refers to a path that is still registered.
+ *
+ * @param subtree subtree from which to start the search, root for initial call
+ * @param path path to subtree (same as _dbus_object_tree_unregister_and_unlock)
+ * @param continue_removal_attempts pointer to a bool, #TRUE for initial call
+ * @param unregister_function_out returns the found node's unregister_function
+ * @param user_data_out returns the found node's user_data
+ * @returns #TRUE if a registered node was found at path, #FALSE otherwise
+ */
+static dbus_bool_t
+unregister_and_free_path_recurse
+(DBusObjectSubtree                 *subtree,
+ const char                       **path,
+ dbus_bool_t                       *continue_removal_attempts,
+ DBusObjectPathUnregisterFunction  *unregister_function_out,
+ void                             **user_data_out)
+{
+  int i, j;
+
+  _dbus_assert (continue_removal_attempts != NULL);
+  _dbus_assert (*continue_removal_attempts);
+  _dbus_assert (unregister_function_out != NULL);
+  _dbus_assert (user_data_out != NULL);
+
+  if (path[0] == NULL)
+    return unregister_subtree (subtree, unregister_function_out, user_data_out);
+
+  i = 0;
+  j = subtree->n_subtrees;
+  while (i < j)
+    {
+      int k, v;
+
+      k = (i + j) / 2;
+      v = strcmp (path[0], subtree->subtrees[k]->name);
+
+      if (v == 0)
+        {
+          dbus_bool_t freed;
+          freed = unregister_and_free_path_recurse (subtree->subtrees[k],
+                                                    &path[1],
+                                                    continue_removal_attempts,
+                                                    unregister_function_out,
+                                                    user_data_out);
+          if (freed && *continue_removal_attempts)
+            *continue_removal_attempts = attempt_child_removal (subtree, k);
+          return freed;
+        }
+      else if (v < 0)
+        {
+          j = k;
+        }
+      else
+        {
+          i = k + 1;
+        }
+    }
+  return FALSE;
+}
+
 /**
  * Unregisters an object subtree that was registered with the
  * same path.
@@ -444,66 +628,42 @@ void
 _dbus_object_tree_unregister_and_unlock (DBusObjectTree          *tree,
                                          const char             **path)
 {
-  int i;
-  DBusObjectSubtree *subtree;
+  dbus_bool_t found_subtree;
+  dbus_bool_t continue_removal_attempts;
   DBusObjectPathUnregisterFunction unregister_function;
   void *user_data;
   DBusConnection *connection;
 
+  _dbus_assert (tree != NULL);
   _dbus_assert (path != NULL);
 
+  continue_removal_attempts = TRUE;
   unregister_function = NULL;
   user_data = NULL;
 
-  subtree = find_subtree (tree, path, &i);
+  found_subtree = unregister_and_free_path_recurse (tree->root,
+                                                    path,
+                                                    &continue_removal_attempts,
+                                                    &unregister_function,
+                                                    &user_data);
 
 #ifndef DBUS_DISABLE_CHECKS
-  if (subtree == NULL)
+  if (found_subtree == FALSE)
     {
-      _dbus_warn ("Attempted to unregister path (path[0] = %s path[1] = %s) which isn't registered\n",
+      _dbus_warn ("Attempted to unregister path (path[0] = %s path[1] = %s) which isn't registered",
                   path[0] ? path[0] : "null",
-                  path[1] ? path[1] : "null");
+                  (path[0] && path[1]) ? path[1] : "null");
       goto unlock;    
     }
 #else
-  _dbus_assert (subtree != NULL);
+  _dbus_assert (found_subtree == TRUE);
 #endif
 
-  _dbus_assert (subtree->parent == NULL ||
-                (i >= 0 && subtree->parent->subtrees[i] == subtree));
-
-  subtree->message_function = NULL;
-
-  unregister_function = subtree->unregister_function;
-  user_data = subtree->user_data;
-
-  subtree->unregister_function = NULL;
-  subtree->user_data = NULL;
-
-  /* If we have no subtrees of our own, remove from
-   * our parent (FIXME could also be more aggressive
-   * and remove our parent if it becomes empty)
-   */
-  if (subtree->parent && subtree->n_subtrees == 0)
-    {
-      /* assumes a 0-byte memmove is OK */
-      memmove (&subtree->parent->subtrees[i],
-               &subtree->parent->subtrees[i+1],
-               (subtree->parent->n_subtrees - i - 1) *
-               sizeof (subtree->parent->subtrees[0]));
-      subtree->parent->n_subtrees -= 1;
-
-      subtree->parent = NULL;
-
-      _dbus_object_subtree_unref (subtree);
-    }
-  subtree = NULL;
-
 unlock:
   connection = tree->connection;
 
   /* Unlock and call application code */
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   if (connection)
 #endif
     {
@@ -515,7 +675,7 @@ unlock:
   if (unregister_function)
     (* unregister_function) (connection, user_data);
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   if (connection)
 #endif
     dbus_connection_unref (connection);
@@ -638,7 +798,7 @@ handle_default_introspect_and_unlock (DBusObjectTree          *tree,
                                     DBUS_INTERFACE_INTROSPECTABLE,
                                     "Introspect"))
     {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
       if (tree->connection)
 #endif
         {
@@ -653,7 +813,7 @@ handle_default_introspect_and_unlock (DBusObjectTree          *tree,
   
   if (!_dbus_string_init (&xml))
     {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
       if (tree->connection)
 #endif
         {
@@ -698,7 +858,7 @@ handle_default_introspect_and_unlock (DBusObjectTree          *tree,
   if (!dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &v_STRING))
     goto out;
   
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   if (tree->connection)
 #endif
     {
@@ -711,7 +871,7 @@ handle_default_introspect_and_unlock (DBusObjectTree          *tree,
   result = DBUS_HANDLER_RESULT_HANDLED;
   
  out:
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   if (tree->connection)
 #endif
     {
@@ -741,11 +901,13 @@ handle_default_introspect_and_unlock (DBusObjectTree          *tree,
  *
  * @param tree the global object tree
  * @param message the message to dispatch
+ * @param found_object return location for the object
  * @returns whether message was handled successfully
  */
 DBusHandlerResult
 _dbus_object_tree_dispatch_and_unlock (DBusObjectTree          *tree,
-                                       DBusMessage             *message)
+                                       DBusMessage             *message,
+                                       dbus_bool_t             *found_object)
 {
   char **path;
   dbus_bool_t exact_match;
@@ -761,7 +923,7 @@ _dbus_object_tree_dispatch_and_unlock (DBusObjectTree          *tree,
   path = NULL;
   if (!dbus_message_get_path_decomposed (message, &path))
     {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
       if (tree->connection)
 #endif
         {
@@ -776,7 +938,7 @@ _dbus_object_tree_dispatch_and_unlock (DBusObjectTree          *tree,
 
   if (path == NULL)
     {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
       if (tree->connection)
 #endif
         {
@@ -791,6 +953,9 @@ _dbus_object_tree_dispatch_and_unlock (DBusObjectTree          *tree,
   /* Find the deepest path that covers the path in the message */
   subtree = find_handler (tree, (const char**) path, &exact_match);
   
+  if (found_object)
+    *found_object = !!subtree;
+
   /* Build a list of all paths that cover the path in the message */
 
   list = NULL;
@@ -842,7 +1007,7 @@ _dbus_object_tree_dispatch_and_unlock (DBusObjectTree          *tree,
           _dbus_verbose ("  (invoking a handler)\n");
 #endif
           
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
           if (tree->connection)
 #endif
             {
@@ -859,7 +1024,7 @@ _dbus_object_tree_dispatch_and_unlock (DBusObjectTree          *tree,
                                          message,
                                          user_data);
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
           if (tree->connection)
 #endif
             _dbus_connection_lock (tree->connection);
@@ -882,7 +1047,7 @@ _dbus_object_tree_dispatch_and_unlock (DBusObjectTree          *tree,
     }
   else
     {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
       if (tree->connection)
 #endif
         {
@@ -1053,7 +1218,7 @@ _dbus_object_tree_list_registered_and_unlock (DBusObjectTree *tree,
                                                        parent_path,
                                                        child_entries);
   
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   if (tree->connection)
 #endif
     {
@@ -1210,7 +1375,7 @@ flatten_path (const char **path)
 }
 
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 
@@ -1391,7 +1556,7 @@ do_test_dispatch (DBusObjectTree *tree,
       ++j;
     }
 
-  result = _dbus_object_tree_dispatch_and_unlock (tree, message);
+  result = _dbus_object_tree_dispatch_and_unlock (tree, message, NULL);
   if (result == DBUS_HANDLER_RESULT_NEED_MEMORY)
     goto oom;
 
@@ -1430,14 +1595,6 @@ do_test_dispatch (DBusObjectTree *tree,
   return FALSE;
 }
 
-static size_t
-string_array_length (const char **array)
-{
-  size_t i;
-  for (i = 0; array[i]; i++) ;
-  return i;
-}
-
 typedef struct
 {
   const char *path;
@@ -1455,6 +1612,8 @@ static DecomposePathTest decompose_tests[] = {
   { "/foo/bar/this/is/longer", { "foo", "bar", "this", "is", "longer", NULL } }
 };
 
+/* Return TRUE on success, FALSE on OOM, die with an assertion failure
+ * on failure. */
 static dbus_bool_t
 run_decompose_tests (void)
 {
@@ -1472,15 +1631,15 @@ run_decompose_tests (void)
                                  &result, &result_len))
         return FALSE;
 
-      expected_len = string_array_length (decompose_tests[i].result);
+      expected_len = _dbus_string_array_length (decompose_tests[i].result);
       
-      if (result_len != (int) string_array_length ((const char**)result) ||
+      if (result_len != (int) _dbus_string_array_length ((const char**)result) ||
           expected_len != result_len ||
           path_contains (decompose_tests[i].result,
                          (const char**) result) != STR_EQUAL)
         {
-          int real_len = string_array_length ((const char**)result);
-          _dbus_warn ("Expected decompose of %s to have len %d, returned %d, appears to have %d\n",
+          int real_len = _dbus_string_array_length ((const char**)result);
+          _dbus_warn ("Expected decompose of %s to have len %d, returned %d, appears to have %d",
                       decompose_tests[i].path, expected_len, result_len,
                       real_len);
           _dbus_warn ("Decompose resulted in elements: { ");
@@ -1491,8 +1650,8 @@ run_decompose_tests (void)
                           (i + 1) == real_len ? "" : ", ");
               ++i;
             }
-          _dbus_warn ("}\n");
-          _dbus_assert_not_reached ("path decompose failed\n");
+          _dbus_warn ("}");
+          _dbus_test_fatal ("path decompose failed");
         }
 
       dbus_free_string_array (result);
@@ -1503,8 +1662,22 @@ run_decompose_tests (void)
   return TRUE;
 }
 
+static DBusObjectSubtree*
+find_subtree_registered_or_unregistered (DBusObjectTree *tree,
+                                         const char    **path)
+{
+#if VERBOSE_FIND
+  _dbus_verbose ("Looking for exact subtree, registered or unregistered\n");
+#endif
+
+  return find_subtree_recurse (tree->root, path, FALSE, NULL, NULL);
+}
+
+/* Returns TRUE if the right thing happens, but the right thing might
+ * be OOM. */
 static dbus_bool_t
-object_tree_test_iteration (void *data)
+object_tree_test_iteration (void        *data,
+                            dbus_bool_t  have_memory)
 {
   const char *path0[] = { NULL };
   const char *path1[] = { "foo", NULL };
@@ -1515,13 +1688,20 @@ object_tree_test_iteration (void *data)
   const char *path6[] = { "blah", "boof", NULL };
   const char *path7[] = { "blah", "boof", "this", "is", "really", "long", NULL };
   const char *path8[] = { "childless", NULL };
+  const char *path9[] = { "blah", "a", NULL };
+  const char *path10[] = { "blah", "b", NULL };
+  const char *path11[] = { "blah", "c", NULL };
+  const char *path12[] = { "blah", "a", "d", NULL };
+  const char *path13[] = { "blah", "b", "d", NULL };
+  const char *path14[] = { "blah", "c", "d", NULL };
+  DBusObjectPathVTable test_vtable = { NULL, test_message_function, NULL };
   DBusObjectTree *tree;
   TreeTestData tree_test_data[9];
   int i;
   dbus_bool_t exact_match;
 
   if (!run_decompose_tests ())
-    return FALSE;
+    return TRUE; /* OOM is OK */
   
   tree = NULL;
 
@@ -1695,7 +1875,7 @@ object_tree_test_iteration (void *data)
     _dbus_object_tree_list_registered_unlocked (tree, path1, &child_entries);
     if (child_entries != NULL)
       {
-	nb = string_array_length ((const char**)child_entries);
+	nb = _dbus_string_array_length ((const char**)child_entries);
 	_dbus_assert (nb == 1);
 	dbus_free_string_array (child_entries);
       }
@@ -1703,7 +1883,7 @@ object_tree_test_iteration (void *data)
     _dbus_object_tree_list_registered_unlocked (tree, path2, &child_entries);
     if (child_entries != NULL)
       {
-	nb = string_array_length ((const char**)child_entries);
+	nb = _dbus_string_array_length ((const char**)child_entries);
 	_dbus_assert (nb == 2);
 	dbus_free_string_array (child_entries);
       }
@@ -1711,7 +1891,7 @@ object_tree_test_iteration (void *data)
     _dbus_object_tree_list_registered_unlocked (tree, path8, &child_entries);
     if (child_entries != NULL)
       {
-	nb = string_array_length ((const char**)child_entries);
+	nb = _dbus_string_array_length ((const char**)child_entries);
 	_dbus_assert (nb == 0);
 	dbus_free_string_array (child_entries);
       }
@@ -1719,7 +1899,7 @@ object_tree_test_iteration (void *data)
     _dbus_object_tree_list_registered_unlocked (tree, root, &child_entries);
     if (child_entries != NULL)
       {
-	nb = string_array_length ((const char**)child_entries);
+	nb = _dbus_string_array_length ((const char**)child_entries);
 	_dbus_assert (nb == 3);
 	dbus_free_string_array (child_entries);
       }
@@ -1885,6 +2065,200 @@ object_tree_test_iteration (void *data)
       ++i;
     }
 
+  /* Test removal of newly-childless unregistered nodes */
+  if (!do_register (tree, path2, TRUE, 2, tree_test_data))
+    goto out;
+
+  _dbus_object_tree_unregister_and_unlock (tree, path2);
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path2));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path1));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path0));
+
+  /* Test that unregistered parents cannot be freed out from under their
+     children */
+  if (!do_register (tree, path2, TRUE, 2, tree_test_data))
+    goto out;
+
+  _dbus_assert (!find_subtree (tree, path1, NULL));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path1));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path0));
+
+#if 0
+  /* This triggers the "Attempted to unregister path ..." warning message */
+  _dbus_object_tree_unregister_and_unlock (tree, path1);
+#endif
+  _dbus_assert (find_subtree (tree, path2, NULL));
+  _dbus_assert (!find_subtree (tree, path1, NULL));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path1));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path0));
+
+  _dbus_object_tree_unregister_and_unlock (tree, path2);
+  _dbus_assert (!find_subtree (tree, path2, NULL));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path2));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path1));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path0));
+
+  /* Test that registered parents cannot be freed out from under their
+     children, and that if they are unregistered before their children, they
+     are still freed when their children are unregistered */
+  if (!do_register (tree, path1, TRUE, 1, tree_test_data))
+    goto out;
+  if (!do_register (tree, path2, TRUE, 2, tree_test_data))
+    goto out;
+
+  _dbus_assert (find_subtree (tree, path1, NULL));
+  _dbus_assert (find_subtree (tree, path2, NULL));
+
+  _dbus_object_tree_unregister_and_unlock (tree, path1);
+  _dbus_assert (!find_subtree (tree, path1, NULL));
+  _dbus_assert (find_subtree (tree, path2, NULL));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path1));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path0));
+
+  _dbus_object_tree_unregister_and_unlock (tree, path2);
+  _dbus_assert (!find_subtree (tree, path1, NULL));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path1));
+  _dbus_assert (!find_subtree (tree, path2, NULL));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path2));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path0));
+
+  /* Test with NULL unregister_function and user_data */
+  if (!_dbus_object_tree_register (tree, TRUE, path2,
+                                   &test_vtable,
+                                   NULL,
+                                   NULL))
+    goto out;
+
+  _dbus_assert (_dbus_object_tree_get_user_data_unlocked (tree, path2) == NULL);
+  _dbus_object_tree_unregister_and_unlock (tree, path2);
+  _dbus_assert (!find_subtree (tree, path2, NULL));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path2));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path1));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path0));
+
+  /* Test freeing a long path */
+  if (!do_register (tree, path3, TRUE, 3, tree_test_data))
+    goto out;
+
+  _dbus_object_tree_unregister_and_unlock (tree, path3);
+  _dbus_assert (!find_subtree (tree, path3, NULL));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path3));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path2));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path1));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path0));
+
+  /* Test freeing multiple children from the same path */
+  if (!do_register (tree, path3, TRUE, 3, tree_test_data))
+    goto out;
+  if (!do_register (tree, path4, TRUE, 4, tree_test_data))
+    goto out;
+
+  _dbus_assert (find_subtree (tree, path3, NULL));
+  _dbus_assert (find_subtree (tree, path4, NULL));
+
+  _dbus_object_tree_unregister_and_unlock (tree, path3);
+  _dbus_assert (!find_subtree (tree, path3, NULL));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path3));
+  _dbus_assert (find_subtree (tree, path4, NULL));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path4));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path2));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path1));
+
+  _dbus_object_tree_unregister_and_unlock (tree, path4);
+  _dbus_assert (!find_subtree (tree, path4, NULL));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path4));
+  _dbus_assert (!find_subtree (tree, path3, NULL));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path3));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path2));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path1));
+
+  /* Test subtree removal */
+  if (!_dbus_object_tree_register (tree, TRUE, path12,
+                                   &test_vtable,
+                                   NULL,
+                                   NULL))
+    goto out;
+
+  _dbus_assert (find_subtree (tree, path12, NULL));
+
+  if (!_dbus_object_tree_register (tree, TRUE, path13,
+                                   &test_vtable,
+                                   NULL,
+                                   NULL))
+    goto out;
+
+  _dbus_assert (find_subtree (tree, path13, NULL));
+
+  if (!_dbus_object_tree_register (tree, TRUE, path14,
+                                   &test_vtable,
+                                   NULL,
+                                   NULL))
+    goto out;
+
+  _dbus_assert (find_subtree (tree, path14, NULL));
+
+  _dbus_object_tree_unregister_and_unlock (tree, path12);
+
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path12));
+  _dbus_assert (find_subtree (tree, path13, NULL));
+  _dbus_assert (find_subtree (tree, path14, NULL));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path9));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path5));
+
+  if (!_dbus_object_tree_register (tree, TRUE, path12,
+                                   &test_vtable,
+                                   NULL,
+                                   NULL))
+    goto out;
+
+  _dbus_assert (find_subtree (tree, path12, NULL));
+
+  _dbus_object_tree_unregister_and_unlock (tree, path13);
+
+  _dbus_assert (find_subtree (tree, path12, NULL));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path13));
+  _dbus_assert (find_subtree (tree, path14, NULL));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path10));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path5));
+
+  if (!_dbus_object_tree_register (tree, TRUE, path13,
+                                   &test_vtable,
+                                   NULL,
+                                   NULL))
+    goto out;
+
+  _dbus_assert (find_subtree (tree, path13, NULL));
+
+  _dbus_object_tree_unregister_and_unlock (tree, path14);
+
+  _dbus_assert (find_subtree (tree, path12, NULL));
+  _dbus_assert (find_subtree (tree, path13, NULL));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path14));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path11));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path5));
+
+  _dbus_object_tree_unregister_and_unlock (tree, path12);
+
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path12));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path9));
+  _dbus_assert (find_subtree_registered_or_unregistered (tree, path5));
+
+  _dbus_object_tree_unregister_and_unlock (tree, path13);
+
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path13));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path10));
+  _dbus_assert (!find_subtree_registered_or_unregistered (tree, path5));
+
+#if 0
+  /* Test attempting to unregister non-existent paths.  These trigger
+     "Attempted to unregister path ..." warning messages */
+  _dbus_object_tree_unregister_and_unlock (tree, path0);
+  _dbus_object_tree_unregister_and_unlock (tree, path1);
+  _dbus_object_tree_unregister_and_unlock (tree, path2);
+  _dbus_object_tree_unregister_and_unlock (tree, path3);
+  _dbus_object_tree_unregister_and_unlock (tree, path4);
+#endif
+
   /* Register it all again, and test dispatch */
   
   if (!do_register (tree, path0, TRUE, 0, tree_test_data))
@@ -1947,15 +2321,13 @@ object_tree_test_iteration (void *data)
  * @returns #TRUE on success.
  */
 dbus_bool_t
-_dbus_object_tree_test (void)
+_dbus_object_tree_test (const char *test_data_dir _DBUS_GNUC_UNUSED)
 {
-  _dbus_test_oom_handling ("object tree",
-                           object_tree_test_iteration,
-                           NULL);
-
-  return TRUE;
+  return _dbus_test_oom_handling ("object tree",
+                                  object_tree_test_iteration,
+                                  NULL);
 }
 
 #endif /* !DOXYGEN_SHOULD_SKIP_THIS */
 
-#endif /* DBUS_BUILD_TESTS */
+#endif /* DBUS_ENABLE_EMBEDDED_TESTS */
