@@ -3,6 +3,8 @@
  *
  * Copyright (C) 2003  Red Hat Inc.
  *
+ * SPDX-License-Identifier: AFL-2.1 OR GPL-2.0-or-later
+ *
  * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,8 +25,8 @@
 #ifndef DBUS_RESOURCES_H
 #define DBUS_RESOURCES_H
 
-#include <dbus/dbus-internals.h>
 #include <dbus/dbus-macros.h>
+#include <dbus/dbus-macros-internal.h>
 #include <dbus/dbus-errors.h>
 #include <dbus/dbus-connection.h>
 
