@@ -463,11 +463,11 @@ DBUS_PRIVATE_EXPORT
 void _dbus_sleep_milliseconds (int milliseconds);
 
 DBUS_PRIVATE_EXPORT
-void _dbus_get_monotonic_time (long *tv_sec,
+void _dbus_get_monotonic_time (dbus_int64_t *tv_sec,
                                long *tv_usec);
 
 DBUS_PRIVATE_EXPORT
-void _dbus_get_real_time (long *tv_sec,
+void _dbus_get_real_time (dbus_int64_t *tv_sec,
                           long *tv_usec);
 
 /**
@@ -493,6 +493,7 @@ dbus_bool_t _dbus_path_is_absolute    (const DBusString *filename);
 
 dbus_bool_t _dbus_get_standard_session_servicedirs (DBusList **dirs);
 dbus_bool_t _dbus_get_standard_system_servicedirs (DBusList **dirs);
+dbus_bool_t _dbus_get_local_system_servicedirs (DBusList **dirs);
 dbus_bool_t _dbus_set_up_transient_session_servicedirs (DBusList  **dirs,
                                                         DBusError  *error);
 
