@@ -2,6 +2,7 @@
 /* dbus-test-tap — TAP helpers for "embedded tests"
  *
  * Copyright © 2017 Collabora Ltd.
+ * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation files
@@ -39,6 +40,7 @@
 #include <stdlib.h>
 
 static unsigned int failures = 0;
+static unsigned int skipped = 0;
 static unsigned int tap_test_counter = 0;
 
 /*
@@ -148,6 +150,7 @@ _dbus_test_skip (const char *format,
   va_list ap;
 
   printf ("ok %u # SKIP ", ++tap_test_counter);
+  ++skipped;
   va_start (ap, format);
   vprintf (format, ap);
   va_end (ap);
@@ -194,10 +197,14 @@ _dbus_test_check_memleaks (const char *test_name)
 int
 _dbus_test_done_testing (void)
 {
+  _dbus_assert (skipped <= tap_test_counter);
+
   if (failures == 0)
-    _dbus_test_diag ("%u tests passed", tap_test_counter);
+    _dbus_test_diag ("%u tests passed (%d skipped)",
+                     tap_test_counter - skipped, skipped);
   else
-    _dbus_test_diag ("%u/%u tests failed", failures, tap_test_counter);
+    _dbus_test_diag ("%u/%u tests failed (%d skipped)",
+                     failures, tap_test_counter - skipped, skipped);
 
   printf ("1..%u\n", tap_test_counter);
   fflush (stdout);
