@@ -38,7 +38,7 @@ if __name__ == "__main__":
     qch = Path(arg_qch)
     qchdir = Path(arg_qchdir)
     destdir = Path(env_destdir)
-    apidir = Path(destdir /docdir / 'api')
+    apidir = Path(destdir / docdir / 'api/html')
     shutil.rmtree(apidir, ignore_errors=True)
     shutil.copytree(builddir / 'api/html', apidir)
     if qch.is_file():
