@@ -660,11 +660,7 @@ _dbus_misc_test (const char *test_data_dir _DBUS_GNUC_UNUSED)
   if (!_dbus_string_init (&str))
     _dbus_test_fatal ("no memory");
 
-  if (!(_dbus_string_append_int (&str, major) &&
-        _dbus_string_append_byte (&str, '.') &&
-        _dbus_string_append_int (&str, minor) &&
-        _dbus_string_append_byte (&str, '.') &&
-        _dbus_string_append_int (&str, micro)))
+  if (!_dbus_string_append_printf (&str, "%d.%d.%d", major, minor, micro))
     _dbus_test_fatal ("no memory");
 
   _dbus_test_check (_dbus_string_equal_c_str (&str, DBUS_VERSION_STRING));
@@ -730,6 +726,8 @@ static char test_socket_dir[] = DBUS_TEST_SOCKET_DIR "/dbus-test-XXXXXX";
 static void
 cleanup_test_socket_tempdir (void)
 {
+  if (chdir ("/tmp") != 0)
+    _dbus_test_fatal ("Failed to chdir() to /tmp");
   if (rmdir (test_socket_dir) != 0)
     _dbus_test_not_ok ("failed to remove test socket directory %s",
                        test_socket_dir);
