@@ -269,6 +269,14 @@ case "$ci_buildsys" in
                 ;;
         esac
 
+        set -- "$@" -D DBUS_BUILD_TESTS=ON
+
+        case "$ci_variant" in
+            (debug)
+                set -- "$@" -D DBUS_ENABLE_INTRUSIVE_TESTS=ON
+                ;;
+        esac
+
         $cmake -DCMAKE_VERBOSE_MAKEFILE=ON -DENABLE_WERROR=ON -S "$srcdir" -B "$ci_builddir" "$@"
 
         ${make}
@@ -354,7 +362,7 @@ case "$ci_buildsys" in
         case "$ci_variant" in
             (debug)
                 set -- -Dasserts=true "$@"
-                set -- -Dembedded_tests=true "$@"
+                set -- -Dintrusive_tests=true "$@"
                 set -- -Dverbose_mode=true "$@"
 
                 case "$ci_host" in
