@@ -3,6 +3,8 @@
  *
  * Copyright (C) 2006 Thiago Macieira <thiago@kde.org>
  *
+ * SPDX-License-Identifier: AFL-2.1 OR GPL-2.0-or-later
+ *
  * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -26,6 +28,8 @@
 
 #include <sys/types.h>
 
+#include <dbus/dbus.h>
+
 #ifndef TRUE
 #define TRUE (1)
 #endif
@@ -40,9 +44,9 @@
 #define MAX_ADDR_LEN 512
 
 /* defined in dbus-launch.c */
-void verbose (const char *format, ...);
+void verbose (const char *format, ...) _DBUS_GNUC_PRINTF (1, 2);
 char *xstrdup (const char *str);
-void kill_bus_and_exit (int exitcode);
+void kill_bus_and_exit (int exitcode) _DBUS_GNUC_NORETURN;
 
 const char* get_machine_uuid (void);
 
